class Ativo {
public:
    Ativo(void);
    Ativo(const Ativo &ativoNeg) { Ativo(); };
    Ativo(string simbolo_);
    ~Ativo(){};

    string getSimbolo();

    double getVolumeMin() { return this.volumeMinimo; }

    double getVolume(double volume_);

    double getPontos() { return this.pontos; }

    double getPontos(double qtdPontos) { return (this.pontos * qtdPontos); }

    int getDigitos() {return (int) SymbolInfoInteger(_Symbol, SYMBOL_DIGITS); }

    double calculoDePontos(void);

    double calculoDePontos(string simbolo_);

    MqlTick getUltimoTick();

    double getLast();

private:
    string simbolo;
    double pontos;
    double volumeMinimo;
    double volumeMaximo;
    MqlTick tick;
    // Método genérico para construtores
    void init();
};

Ativo::Ativo(void){
    simbolo = _Symbol;
    this.init();
}

Ativo::Ativo(string simbolo_){
    simbolo = simbolo_;
    this.init();
}

string Ativo::getSimbolo(){
    if(this.simbolo == "")
        this.simbolo = _Symbol;
    return this.simbolo;
}

double Ativo::getVolume(double volume_){
    double volumeMin = this.getVolumeMin();
    //Corrige o volume inserido subtraindo pelo resto da divisão do volume mínimo
    double volumeCorrigido = volume_ - (MathMod(volume_, volumeMin));
    double volumeRet;

    if(volumeMin > 1) //Se for um ativo com lote mínimo maior que 1, multiplica pelo lote mínimo
        volumeRet = volumeMin * volumeCorrigido;
    else //Se não for maior que 1, é um ativo de n variações de lotes dentro do range de 0.01 a 1
        volumeRet = volumeCorrigido;

    //Valida se o volume de retorno é maior ou igual ao mínimo, se não for, devolve o mínimo
    if(volumeRet >= volumeMin)
            return volumeRet;
        else
            return volumeMin;
}

double Ativo::calculoDePontos(void){
    return (this.calculoDePontos(this.getSimbolo()));
}

double Ativo::calculoDePontos(string simbolo_){
    double valorTick = SymbolInfoDouble(simbolo_, SYMBOL_TRADE_TICK_VALUE);
    double tamanhoTick = SymbolInfoDouble(simbolo_, SYMBOL_TRADE_TICK_SIZE);

    double volumeMin = getVolumeMin();
    double qtdTicksParaUmPonto;

    if(volumeMin < 100) // Divisão de 10 pelo volume/valor do tick (Lucro por tick), pois 10 de lucro é a referência para 1 ponto
        qtdTicksParaUmPonto= 10 / (valorTick / volumeMin);
    else
        qtdTicksParaUmPonto= 10 / (volumeMin * valorTick);
    // Multiplica o tamanho do tick pela quantidade calculada, para retornar o valor em pontos do ativo
    return (tamanhoTick * qtdTicksParaUmPonto);
}

void Ativo::init(){
    volumeMinimo = SymbolInfoDouble(simbolo, SYMBOL_VOLUME_MIN);
    volumeMaximo = SymbolInfoDouble(simbolo, SYMBOL_VOLUME_MAX);
    pontos = calculoDePontos();
}

MqlTick Ativo::getUltimoTick(){
    SymbolInfoTick(this.getSimbolo(), this.tick);
    return this.tick;
}

double Ativo::getLast(){
    MqlTick localTick = this.getUltimoTick();
    if(localTick.last > 0)
        return localTick.last;
    else if(localTick.ask > 0)
        return localTick.ask;
    else
        return localTick.bid;
}